@@ -21,16 +21,21 @@ from neutron.common import config as n_conf
 from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
 from neutron_lib import worker
+from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
 
 from networking_ovn.common import constants as ovn_const
 from networking_ovn.db import maintenance as db_maint
 from networking_ovn.db import revision as db_rev
+from networking_ovn import ovn_db_sync
 
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 DB_CONSISTENCY_CHECK_INTERVAL = 300  # 5 minutes
+INCONSISTENCY_TYPE_CREATE_UPDATE = 'create/update'
+INCONSISTENCY_TYPE_DELETE = 'delete'
 
 
 class MaintenanceWorker(worker.BaseWorker):
@@ -216,6 +221,57 @@ class DBInconsistenciesPeriodics(object):
         else:
             self._ovn_client.update_subnet(sn_db_obj, n_db_obj)
 
+    # The migration will run just once per neutron-server instance. If the lock
+    # is held by some other neutron-server instance in the cloud, we'll attempt
+    # to perform the migration every 10 seconds until completed.
+    @periodics.periodic(spacing=10, run_immediately=True)
+    def migrate_to_port_groups(self):
+        """Perform the migration from Address Sets to Port Groups. """
+        # TODO(dalvarez): Remove this in U cycle when we're sure that all
+        # versions are running using Port Groups (and OVS >= 2.10).
+
+        # If Port Groups are not supported or we've already migrated, we don't
+        # need to attempt to migrate again.
+        if (not self._nb_idl.is_port_groups_supported() or
+                not self._nb_idl.get_address_sets()):
+            raise periodics.NeverAgain()
+
+        # Only the worker holding a valid lock within OVSDB will perform the
+        # migration.
+        if not self.has_lock:
+            return
+
+        admin_context = n_context.get_admin_context()
+        nb_sync = ovn_db_sync.OvnNbSynchronizer(
+            self._ovn_client._plugin, self._nb_idl, self._ovn_client._sb_idl,
+            None, None)
+        nb_sync.migrate_to_port_groups(admin_context)
+        raise periodics.NeverAgain()
+
+    def _log_maintenance_inconsistencies(self, create_update_inconsistencies,
+                                         delete_inconsistencies):
+        if not CONF.debug:
+            return
+
+        def _log(inconsistencies, type_):
+            if not inconsistencies:
+                return
+
+            c = {}
+            for f in inconsistencies:
+                if f.resource_type not in c:
+                    c[f.resource_type] = 1
+                else:
+                    c[f.resource_type] += 1
+
+            fail_str = ', '.join('{}={}'.format(k, v) for k, v in c.items())
+            LOG.debug('Maintenance task: Number of inconsistencies '
+                      'found at %(type_)s: %(fail_str)s',
+                      {'type_': type_, 'fail_str': fail_str})
+
+        _log(create_update_inconsistencies, INCONSISTENCY_TYPE_CREATE_UPDATE)
+        _log(delete_inconsistencies, INCONSISTENCY_TYPE_DELETE)
+
     @periodics.periodic(spacing=DB_CONSISTENCY_CHECK_INTERVAL,
                         run_immediately=True)
     def check_for_inconsistencies(self):
@@ -227,13 +283,22 @@ class DBInconsistenciesPeriodics(object):
         create_update_inconsistencies = db_maint.get_inconsistent_resources()
         delete_inconsistencies = db_maint.get_deleted_resources()
         if not any([create_update_inconsistencies, delete_inconsistencies]):
+            LOG.debug('Maintenance task: No inconsistencies found. Skipping')
             return
+
         LOG.debug('Maintenance task: Synchronizing Neutron '
                   'and OVN databases')
+        self._log_maintenance_inconsistencies(create_update_inconsistencies,
+                                              delete_inconsistencies)
         self._sync_timer.restart()
 
+        dbg_log_msg = ('Maintenance task: Fixing resource %(res_uuid)s '
+                       '(type: %(res_type)s) at %(type_)s')
         # Fix the create/update resources inconsistencies
         for row in create_update_inconsistencies:
+            LOG.debug(dbg_log_msg, {'res_uuid': row.resource_uuid,
+                                    'res_type': row.resource_type,
+                                    'type_': INCONSISTENCY_TYPE_CREATE_UPDATE})
             try:
                 # NOTE(lucasagomes): The way to fix subnets is bit
                 # different than other resources. A subnet in OVN language
@@ -246,26 +311,29 @@ class DBInconsistenciesPeriodics(object):
                 else:
                     self._fix_create_update(row)
             except Exception:
-                LOG.exception('Failed to fix resource %(res_uuid)s '
-                              '(type: %(res_type)s)',
+                LOG.exception('Maintenance task: Failed to fix resource '
+                              '%(res_uuid)s (type: %(res_type)s)',
                               {'res_uuid': row.resource_uuid,
                                'res_type': row.resource_type})
 
         # Fix the deleted resources inconsistencies
         for row in delete_inconsistencies:
+            LOG.debug(dbg_log_msg, {'res_uuid': row.resource_uuid,
+                                    'res_type': row.resource_type,
+                                    'type_': INCONSISTENCY_TYPE_DELETE})
             try:
                 if row.resource_type == ovn_const.TYPE_SUBNETS:
                     self._ovn_client.delete_subnet(row.resource_uuid)
                 else:
                     self._fix_delete(row)
             except Exception:
-                LOG.exception('Failed to fix deleted resource %(res_uuid)s '
-                              '(type: %(res_type)s)',
+                LOG.exception('Maintenance task: Failed to fix deleted '
+                              'resource %(res_uuid)s (type: %(res_type)s)',
                               {'res_uuid': row.resource_uuid,
                                'res_type': row.resource_type})
 
         self._sync_timer.stop()
-        LOG.info('Maintenance task synchronization finished '
+        LOG.info('Maintenance task: Synchronization finished '
                  '(took %.2f seconds)', self._sync_timer.elapsed())
 
     def _create_lrouter_port(self, port):
@@ -273,3 +341,32 @@ class DBInconsistenciesPeriodics(object):
         router_id = port['device_id']
         self._ovn_client._l3_plugin.add_router_interface(
             admin_context, router_id, {'port_id': port['id']}, may_exist=True)
+
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def check_for_port_security_unknown_address(self):
+
+        if not self.has_lock:
+            return
+
+        for port in self._nb_idl.lsp_list().execute(check_error=True):
+
+            if port.type == ovn_const.LSP_TYPE_LOCALNET:
+                continue
+
+            addresses = port.addresses
+            type_ = port.type.strip()
+            if not port.port_security:
+                if not type_ and ovn_const.UNKNOWN_ADDR not in addresses:
+                    addresses.append(ovn_const.UNKNOWN_ADDR)
+                elif type_ and ovn_const.UNKNOWN_ADDR in addresses:
+                    addresses.remove(ovn_const.UNKNOWN_ADDR)
+            else:
+                if type_ and ovn_const.UNKNOWN_ADDR in addresses:
+                    addresses.remove(ovn_const.UNKNOWN_ADDR)
+                elif not type_ and ovn_const.UNKNOWN_ADDR in addresses:
+                    addresses.remove(ovn_const.UNKNOWN_ADDR)
+
+            self._nb_idl.lsp_set_addresses(
+                port.name, addresses=addresses).execute(check_error=True)
+
+        raise periodics.NeverAgain()
