@@ -53,7 +53,8 @@ OvnPortInfo = collections.namedtuple(
 
 
 GW_INFO = collections.namedtuple('GatewayInfo', ['network_id', 'subnet_id',
-                                                 'router_ip', 'gateway_ip'])
+                                                 'router_ip', 'gateway_ip',
+                                                 'ip_version', 'ip_prefix'])
 
 
 class OVNClient(object):
@@ -196,13 +197,14 @@ class OVNClient(object):
             qos_options = self._qos_driver.get_qos_options(port)
         vtep_physical_switch = binding_prof.get('vtep-physical-switch')
 
+        port_type = ''
         cidrs = ''
         if vtep_physical_switch:
             vtep_logical_switch = binding_prof.get('vtep-logical-switch')
             port_type = 'vtep'
             options = {'vtep-physical-switch': vtep_physical_switch,
                        'vtep-logical-switch': vtep_logical_switch}
-            addresses = ["unknown"]
+            addresses = [ovn_const.UNKNOWN_ADDR]
             parent_name = []
             tag = []
             port_security = []
@@ -212,17 +214,34 @@ class OVNClient(object):
             tag = binding_prof.get('tag', [])
             address = port['mac_address']
             for ip in port.get('fixed_ips', []):
+                try:
+                    subnet = self._plugin.get_subnet(
+                        n_context.get_admin_context(), ip['subnet_id'])
+                except n_exc.SubnetNotFound:
+                    continue
                 address += ' ' + ip['ip_address']
-                subnet = self._plugin.get_subnet(n_context.get_admin_context(),
-                                                 ip['subnet_id'])
                 cidrs += ' {}/{}'.format(ip['ip_address'],
                                          subnet['cidr'].split('/')[1])
             port_security, new_macs = \
                 self._get_allowed_addresses_from_port(port)
             addresses = [address]
             addresses.extend(new_macs)
-            port_type = ovn_const.OVN_NEUTRON_OWNER_TO_PORT_TYPE.get(
-                port['device_owner'], '')
+
+            # Only adjust the OVN type if the port is not owned by Neutron
+            # DHCP agents.
+            if (port['device_owner'] == const.DEVICE_OWNER_DHCP and
+                    not utils.is_neutron_dhcp_agent_port(port)):
+                port_type = 'localport'
+
+            # The "unknown" address should only be set for the normal LSP
+            # ports (the ones which type is empty)
+            if not port_security and not port_type:
+                # Port security is disabled for this port.
+                # So this port can send traffic with any mac address.
+                # OVN allows any mac address from a port if "unknown"
+                # is added to the Logical_Switch_Port.addresses column.
+                # So add it.
+                addresses.append(ovn_const.UNKNOWN_ADDR)
 
         dhcpv4_options = self._get_port_dhcp_options(port, const.IP_VERSION_4)
         dhcpv6_options = self._get_port_dhcp_options(port, const.IP_VERSION_6)
@@ -283,19 +302,33 @@ class OVNClient(object):
             # The lport_name *must* be neutron port['id'].  It must match the
             # iface-id set in the Interfaces table of the Open_vSwitch
             # database which nova sets to be the port ID.
+
+            kwargs = {
+                'lport_name': port['id'],
+                'lswitch_name': lswitch_name,
+                'addresses': port_info.addresses,
+                'external_ids': external_ids,
+                'parent_name': port_info.parent_name,
+                'tag': port_info.tag,
+                'enabled': port.get('admin_state_up'),
+                'options': port_info.options,
+                'type': port_info.type,
+                'port_security': port_info.port_security,
+                'dhcpv4_options': dhcpv4_options,
+                'dhcpv6_options': dhcpv6_options
+            }
+
+            # NOTE(mjozefcz): Do not set addresses if the port is not
+            # bound and has no device_owner - possibly it is a VirtualIP
+            # port used for Octavia (VRRP).
+            # For more details check related bug #1789686.
+            if (not port.get('device_owner') and
+                port.get(portbindings.VIF_TYPE) ==
+                    portbindings.VIF_TYPE_UNBOUND):
+                kwargs['addresses'] = []
+
             port_cmd = txn.add(self._nb_idl.create_lswitch_port(
-                lport_name=port['id'],
-                lswitch_name=lswitch_name,
-                addresses=port_info.addresses,
-                external_ids=external_ids,
-                parent_name=port_info.parent_name,
-                tag=port_info.tag,
-                enabled=port.get('admin_state_up'),
-                options=port_info.options,
-                type=port_info.type,
-                port_security=port_info.port_security,
-                dhcpv4_options=dhcpv4_options,
-                dhcpv6_options=dhcpv6_options))
+                **kwargs))
 
             # Handle ACL's for this port. If we're not using Port Groups
             # because either the schema doesn't support it or we didn't
@@ -315,7 +348,7 @@ class OVNClient(object):
                     txn.add(self._nb_idl.pg_add_ports(
                         utils.ovn_port_group_name(sg), port_cmd))
             else:
-                # SGs modelled as Port Groups:
+                # SGs modelled as Address Sets:
                 acls_new = ovn_acl.add_acls(self._plugin, admin_context,
                                             port, sg_cache, subnet_cache,
                                             self._nb_idl)
@@ -400,6 +433,16 @@ class OVNClient(object):
                 dhcpv6_options = txn.add(port_info.dhcpv6_options['cmd'])
             else:
                 dhcpv6_options = [port_info.dhcpv6_options['uuid']]
+
+            # NOTE(mjozefcz): Do not set addresses if the port is not
+            # bound and has no device_owner - possibly it is a VirtualIP
+            # port used for Octavia (VRRP).
+            # For more details check related bug #1789686.
+            if (not port.get('device_owner') and
+                port.get(portbindings.VIF_TYPE) ==
+                    portbindings.VIF_TYPE_UNBOUND):
+                columns_dict['addresses'] = []
+
             # NOTE(lizk): Fail port updating if port doesn't exist. This
             # prevents any new inserted resources to be orphan, such as port
             # dhcp options or ACL rules for port, e.g. a port was created
@@ -453,9 +496,11 @@ class OVNClient(object):
                         utils.is_port_security_enabled(port)):
                     self._add_port_to_drop_port_group(port['id'], txn)
                 # If the port doesn't belong to any security group and
-                # port_security is disabled, allow all traffic
-                elif (not new_sg_ids and
-                      not utils.is_port_security_enabled(port)):
+                # port_security is disabled, or it's a trusted port, then
+                # allow all traffic.
+                elif ((not new_sg_ids and
+                      not utils.is_port_security_enabled(port)) or
+                      utils.is_lsp_trusted(port)):
                     self._del_port_from_drop_port_group(port['id'], txn)
             else:
                 # Refresh ACLs for changed security groups or fixed IPs.
@@ -582,9 +627,7 @@ class OVNClient(object):
         context = n_context.get_admin_context()
         fip_db = self._l3_plugin._get_floatingip(context, floatingip['id'])
 
-        func = self._nb_idl.add_nat_rule_in_lrouter
         gw_lrouter_name = utils.ovn_name(router_id)
-        nat_rule_args = (gw_lrouter_name,)
         # TODO(chandrav): Since the floating ip port is not
         # bound to any chassis, packets destined to floating ip
         # will be dropped. To overcome this, delete the floating
@@ -595,21 +638,6 @@ class OVNClient(object):
                         fip_db['floating_port_id'],
                         utils.ovn_name(floatingip['floating_network_id'])))
 
-        # Get the list of nat rules and check if the external_ip
-        # with type 'dnat_and_snat' already exists or not.
-        # If exists, set the new value.
-        # This happens when the port associated to a floating ip
-        # is deleted before the disassociation.
-        lrouter_nat_rules = self._nb_idl.get_lrouter_nat_rules(
-            gw_lrouter_name)
-        for nat_rule in lrouter_nat_rules:
-            if (nat_rule['external_ip'] ==
-                    floatingip['floating_ip_address'] and
-                    nat_rule['type'] == 'dnat_and_snat'):
-                func = self._nb_idl.set_nat_rule_in_lrouter
-                nat_rule_args = (gw_lrouter_name, nat_rule['uuid'])
-                break
-
         ext_ids = {
             ovn_const.OVN_FIP_EXT_ID_KEY: floatingip['id'],
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(utils.get_revision_number(
@@ -626,14 +654,36 @@ class OVNClient(object):
             columns['logical_port'] = floatingip['port_id']
             ext_ids[ovn_const.OVN_FIP_EXT_MAC_KEY] = port['mac_address']
             if self._nb_idl.lsp_get_up(floatingip['port_id']).execute():
-                columns['external_mac'] = port['mac_address']
+                port_net = self._plugin.get_network(context,
+                                                    port['network_id'])
+                if port_net.get(pnet.NETWORK_TYPE) != const.TYPE_VLAN:
+                    columns['external_mac'] = port['mac_address']
 
         # TODO(dalvarez): remove this check once the minimum OVS required
         # version contains the column (when OVS 2.8.2 is released).
         if self._nb_idl.is_col_present('NAT', 'external_ids'):
             columns['external_ids'] = ext_ids
 
-        commands.append(func(*nat_rule_args, **columns))
+        commands.append(self._nb_idl.add_nat_rule_in_lrouter(gw_lrouter_name,
+                                                             **columns))
+
+        # Get the logical port (of the private network) and set the field
+        # external_ids:fip=<FIP>. This will be used by the ovn octavia driver
+        # to add the floating ip as vip in the Load_Balancer.vips column.
+        private_lsp = self._nb_idl.get_lswitch_port(floatingip['port_id'])
+
+        if private_lsp:
+            port_fip = {
+                ovn_const.OVN_PORT_FIP_EXT_ID_KEY:
+                    floatingip['floating_ip_address']}
+            commands.append(
+                self._nb_idl.db_set('Logical_Switch_Port', private_lsp.uuid,
+                                    ('external_ids', port_fip))
+            )
+        else:
+            LOG.warning("LSP for floatingip %s, has not been found! "
+                        "Cannot set FIP on VIP.",
+                        floatingip['id'])
         self._transaction(commands, txn=txn)
 
     def _delete_floatingip(self, fip, lrouter, txn=None):
@@ -641,6 +691,20 @@ class OVNClient(object):
                     lrouter, type='dnat_and_snat',
                     logical_ip=fip['logical_ip'],
                     external_ip=fip['external_ip'])]
+        try:
+            port_id = (
+                fip['external_ids'].get(ovn_const.OVN_FIP_PORT_EXT_ID_KEY))
+            if port_id:
+                private_lsp = self._nb_idl.get_lswitch_port(port_id)
+                if private_lsp:
+                    commands.append(
+                        self._nb_idl.db_remove(
+                            'Logical_Switch_Port', private_lsp.uuid,
+                            'external_ids',
+                            (ovn_const.OVN_PORT_FIP_EXT_ID_KEY))
+                    )
+        except KeyError:
+            LOG.debug("FIP %s doesn't have external_ids.", fip)
         self._transaction(commands, txn=txn)
 
     def update_floatingip_status(self, floatingip):
@@ -691,15 +755,10 @@ class OVNClient(object):
             floatingip['id'], floatingip, ovn_const.TYPE_FLOATINGIPS)
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(check_rev_cmd)
-            if (ovn_fip and
-                (floatingip['fixed_ip_address'] != ovn_fip['logical_ip'] or
-                 floatingip['port_id'] != ovn_fip['external_ids'].get(
-                    ovn_const.OVN_FIP_PORT_EXT_ID_KEY))):
-
+            if ovn_fip:
                 lrouter = ovn_fip['external_ids'].get(
                     ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY,
                     utils.ovn_name(router_id))
-
                 self._delete_floatingip(ovn_fip, lrouter, txn=txn)
                 fip_status = const.FLOATINGIP_STATUS_DOWN
 
@@ -750,16 +809,18 @@ class OVNClient(object):
                           'router. Error: %s', e)
 
     def _get_gw_info(self, context, router):
+        gateways_info = []
         ext_gw_info = router.get(l3.EXTERNAL_GW_INFO, {})
         network_id = ext_gw_info.get('network_id', '')
         for ext_fixed_ip in ext_gw_info.get('external_fixed_ips', []):
             subnet_id = ext_fixed_ip['subnet_id']
             subnet = self._plugin.get_subnet(context, subnet_id)
-            if subnet['ip_version'] == 4:
-                return GW_INFO(
-                    network_id, subnet_id, ext_fixed_ip['ip_address'],
-                    subnet.get('gateway_ip'))
-        return GW_INFO('', '', '', '')
+            gateways_info.append(GW_INFO(
+                network_id, subnet_id, ext_fixed_ip['ip_address'],
+                subnet.get('gateway_ip'), subnet['ip_version'],
+                const.IPv4_ANY if subnet['ip_version'] == const.IP_VERSION_4
+                else const.IPv6_ANY))
+        return gateways_info
 
     def _delete_router_ext_gw(self, context, router, networks, txn):
         if not networks:
@@ -767,17 +828,19 @@ class OVNClient(object):
         router_id = router['id']
         gw_port_id = router['gw_port_id']
         gw_lrouter_name = utils.ovn_name(router_id)
-        gw_info = self._get_gw_info(context, router)
-        txn.add(self._nb_idl.delete_static_route(gw_lrouter_name,
-                                                 ip_prefix='0.0.0.0/0',
-                                                 nexthop=gw_info.gateway_ip))
+        gateways = self._get_gw_info(context, router)
+        for gw_info in gateways:
+            if gw_info.ip_version == const.IP_VERSION_4:
+                for network in networks:
+                    txn.add(self._nb_idl.delete_nat_rule_in_lrouter(
+                        gw_lrouter_name, type='snat', logical_ip=network,
+                        external_ip=gw_info.router_ip))
+            txn.add(self._nb_idl.delete_static_route(
+                gw_lrouter_name, ip_prefix=gw_info.ip_prefix,
+                nexthop=gw_info.gateway_ip))
         txn.add(self._nb_idl.delete_lrouter_port(
             utils.ovn_lrouter_port_name(gw_port_id),
             gw_lrouter_name))
-        for network in networks:
-            txn.add(self._nb_idl.delete_nat_rule_in_lrouter(
-                gw_lrouter_name, type='snat', logical_ip=network,
-                external_ip=gw_info.router_ip))
 
     def _get_nets_and_ipv6_ra_confs_for_router_port(
             self, port_fixed_ips):
@@ -808,45 +871,55 @@ class OVNClient(object):
     def _add_router_ext_gw(self, context, router, networks, txn):
         router_id = router['id']
         # 1. Add the external gateway router port.
-        gw_info = self._get_gw_info(context, router)
+        gateways = self._get_gw_info(context, router)
         gw_port_id = router['gw_port_id']
         port = self._plugin.get_port(context, gw_port_id)
         self._create_lrouter_port(router_id, port, txn=txn)
 
-        # TODO(lucasagomes): Remove this check after OVS 2.8.2 is tagged
-        # (prior to that, the external_ids column didn't exist in this
-        # table).
-        columns = {}
-        if self._nb_idl.is_col_present('Logical_Router_Static_Route',
-                                       'external_ids'):
-            columns['external_ids'] = {
-                ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
-                ovn_const.OVN_SUBNET_EXT_ID_KEY: gw_info.subnet_id}
+        def _build_extids(gw_info):
+            # TODO(lucasagomes): Remove this check after OVS 2.8.2 is tagged
+            # (prior to that, the external_ids column didn't exist in this
+            # table).
+            columns = {}
+            if self._nb_idl.is_col_present('Logical_Router_Static_Route',
+                                           'external_ids'):
+                columns['external_ids'] = {
+                    ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
+                    ovn_const.OVN_SUBNET_EXT_ID_KEY: gw_info.subnet_id}
+            return columns
 
         # 2. Add default route with nexthop as gateway ip
         lrouter_name = utils.ovn_name(router_id)
-        txn.add(self._nb_idl.add_static_route(
-            lrouter_name, ip_prefix='0.0.0.0/0', nexthop=gw_info.gateway_ip,
-            **columns))
+        for gw_info in gateways:
+            columns = _build_extids(gw_info)
+            txn.add(self._nb_idl.add_static_route(
+                lrouter_name, ip_prefix=gw_info.ip_prefix,
+                nexthop=gw_info.gateway_ip, **columns))
 
         # 3. Add snat rules for tenant networks in lrouter if snat is enabled
         if utils.is_snat_enabled(router) and networks:
             self.update_nat_rules(router, networks, enable_snat=True, txn=txn)
         return port
 
-    def _check_external_ips_changed(self, context, ovn_snats, ovn_static_route,
-                                    router):
-        gw_info = self._get_gw_info(context, router)
+    def _check_external_ips_changed(self, context, ovn_snats,
+                                    ovn_static_routes, router):
+        gateways = self._get_gw_info(context, router)
+        ovn_gw_subnets = None
         if self._nb_idl.is_col_present('Logical_Router_Static_Route',
                                        'external_ids'):
-            ext_ids = getattr(ovn_static_route, 'external_ids', {})
-            if (ext_ids.get(ovn_const.OVN_SUBNET_EXT_ID_KEY) !=
-               gw_info.subnet_id):
-                return True
+            ovn_gw_subnets = [
+                getattr(route, 'external_ids', {}).get(
+                    ovn_const.OVN_SUBNET_EXT_ID_KEY) for route in
+                ovn_static_routes]
 
-        for snat in ovn_snats:
-            if snat.external_ip != gw_info.router_ip:
+        for gw_info in gateways:
+            if ovn_gw_subnets and gw_info.subnet_id not in ovn_gw_subnets:
                 return True
+            if gw_info.ip_version == 6:
+                continue
+            for snat in ovn_snats:
+                if snat.external_ip != gw_info.router_ip:
+                    return True
 
         return False
 
@@ -1071,7 +1144,9 @@ class OVNClient(object):
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(utils.get_revision_number(
                 port, ovn_const.TYPE_ROUTER_PORTS)),
             ovn_const.OVN_SUBNET_EXT_IDS_KEY:
-                ' '.join(utils.get_port_subnet_ids(port))}
+                ' '.join(utils.get_port_subnet_ids(port)),
+            ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
+                utils.ovn_name(port['network_id'])}
 
         router_id = port.get('device_id')
         if router_id:
@@ -1290,27 +1365,28 @@ class OVNClient(object):
         func = (self._nb_idl.add_nat_rule_in_lrouter if enable_snat else
                 self._nb_idl.delete_nat_rule_in_lrouter)
         gw_lrouter_name = utils.ovn_name(router['id'])
-        gw_info = self._get_gw_info(context, router)
-        commands = []
-        for network in networks:
-            commands.append(
-                func(gw_lrouter_name, type='snat', logical_ip=network,
-                     external_ip=gw_info.router_ip))
+        gateways = self._get_gw_info(context, router)
+        # Update NAT rules only for IPv4 subnets
+        commands = [func(gw_lrouter_name, type='snat', logical_ip=network,
+                         external_ip=gw_info.router_ip) for gw_info in gateways
+                    if gw_info.ip_version != const.IP_VERSION_6
+                    for network in networks]
         self._transaction(commands, txn=txn)
 
     def _create_provnet_port(self, txn, network, physnet, tag):
         txn.add(self._nb_idl.create_lswitch_port(
             lport_name=utils.ovn_provnet_port_name(network['id']),
             lswitch_name=utils.ovn_name(network['id']),
-            addresses=['unknown'],
+            addresses=[ovn_const.UNKNOWN_ADDR],
             external_ids={},
-            type='localnet',
+            type=ovn_const.LSP_TYPE_LOCALNET,
             tag=tag if tag else [],
             options={'network_name': physnet}))
 
     def _gen_network_external_ids(self, network):
         ext_ids = {
             ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY: network['name'],
+            ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY: str(network['mtu']),
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(
                 utils.get_revision_number(network, ovn_const.TYPE_NETWORKS))}
 
@@ -1403,7 +1479,19 @@ class OVNClient(object):
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(check_rev_cmd)
             ext_ids = self._gen_network_external_ids(network)
+            lswitch = self._nb_idl.get_lswitch(lswitch_name)
             txn.add(self._nb_idl.set_lswitch_ext_ids(lswitch_name, ext_ids))
+            # Check if previous mtu is different than current one,
+            # checking will help reduce number of operations
+            if (not lswitch or
+                    lswitch.external_ids.get(
+                        ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY) !=
+                    str(network['mtu'])):
+                context = n_context.get_admin_context()
+                subnets = self._plugin.get_subnets_by_network(
+                    context, network['id'])
+                for subnet in subnets:
+                    self.update_subnet(subnet, network, txn)
 
         if check_rev_cmd.result == ovn_const.TXN_COMMITTED:
             if qos_update_required:
@@ -1623,30 +1711,17 @@ class OVNClient(object):
                 self.update_metadata_port(context, network['id'])
 
             self._add_subnet_dhcp_options(subnet, network)
-
-            if self._nb_idl.is_port_groups_supported():
-                with self._nb_idl.transaction(check_error=True) as txn:
-                    self._create_subnet_port_group(subnet, txn)
-
         db_rev.bump_revision(subnet, ovn_const.TYPE_SUBNETS)
 
-    def _create_subnet_port_group(self, subnet, txn):
-        pg_name = utils.ovn_port_group_name(subnet['id'])
-        if self._nb_idl.get_port_group(pg_name):
-            return
-        # Create subnet Port Group.
-        txn.add(self._nb_idl.pg_add(pg_name, acls=[]))
-        # Add ACLs to this Port Group.
-        acls = ovn_acl.add_acls_for_subnet_port_group(
-            self._nb_idl, pg_name, subnet)
-        for acl in acls:
-            txn.add(self._nb_idl.pg_acl_add(**acl))
+    def _modify_subnet_dhcp_options(self, subnet, ovn_subnet, network, txn):
+        if subnet['enable_dhcp'] and not ovn_subnet:
+            self._enable_subnet_dhcp_options(subnet, network, txn)
+        elif subnet['enable_dhcp'] and ovn_subnet:
+            self._update_subnet_dhcp_options(subnet, network, txn)
+        elif not subnet['enable_dhcp'] and ovn_subnet:
+            self._remove_subnet_dhcp_options(subnet['id'], txn)
 
-    def _delete_subnet_port_group(self, subnet_id, txn):
-        pg_name = utils.ovn_port_group_name(subnet_id)
-        txn.add(self._nb_idl.pg_del(pg_name, if_exists=True))
-
-    def update_subnet(self, subnet, network):
+    def update_subnet(self, subnet, network, txn=None):
         ovn_subnet = self._nb_idl.get_subnet_dhcp_options(
             subnet['id'])['subnet']
 
@@ -1656,27 +1731,19 @@ class OVNClient(object):
 
         check_rev_cmd = self._nb_idl.check_revision_number(
             subnet['id'], subnet, ovn_const.TYPE_SUBNETS)
-        with self._nb_idl.transaction(check_error=True) as txn:
-            txn.add(check_rev_cmd)
-            if subnet['enable_dhcp'] and not ovn_subnet:
-                self._enable_subnet_dhcp_options(subnet, network, txn)
-                if self._nb_idl.is_port_groups_supported():
-                    self._create_subnet_port_group(subnet, txn)
-            elif not subnet['enable_dhcp'] and ovn_subnet:
-                self._remove_subnet_dhcp_options(subnet['id'], txn)
-                if self._nb_idl.is_port_groups_supported():
-                    self._delete_subnet_port_group(subnet, txn)
-            elif subnet['enable_dhcp'] and ovn_subnet:
-                self._update_subnet_dhcp_options(subnet, network, txn)
-
+        if not txn:
+            with self._nb_idl.transaction(check_error=True) as txn_n:
+                txn_n.add(check_rev_cmd)
+                self._modify_subnet_dhcp_options(subnet, ovn_subnet, network,
+                                                 txn_n)
+        else:
+            self._modify_subnet_dhcp_options(subnet, ovn_subnet, network, txn)
         if check_rev_cmd.result == ovn_const.TXN_COMMITTED:
             db_rev.bump_revision(subnet, ovn_const.TYPE_SUBNETS)
 
     def delete_subnet(self, subnet_id):
         with self._nb_idl.transaction(check_error=True) as txn:
             self._remove_subnet_dhcp_options(subnet_id, txn)
-            if self._nb_idl.is_port_groups_supported():
-                self._delete_subnet_port_group(subnet_id, txn)
         db_rev.delete_revision(subnet_id, ovn_const.TYPE_SUBNETS)
 
     def create_security_group(self, security_group):
@@ -1707,11 +1774,11 @@ class OVNClient(object):
         with self._nb_idl.transaction(check_error=True) as txn:
             if not self._nb_idl.get_port_group(pg_name):
                 # If drop Port Group doesn't exist yet, create it.
-                txn.add(self._nb_idl.pg_add(pg_name, acls=[]))
+                txn.add(self._nb_idl.pg_add(pg_name, acls=[], may_exist=True))
                 # Add ACLs to this Port Group so that all traffic is dropped.
                 acls = ovn_acl.add_acls_for_drop_port_group(pg_name)
                 for acl in acls:
-                    txn.add(self._nb_idl.pg_acl_add(**acl))
+                    txn.add(self._nb_idl.pg_acl_add(may_exist=True, **acl))
 
             if ports:
                 ports_ids = [port['id'] for port in ports]
@@ -1761,9 +1828,12 @@ class OVNClient(object):
 
         ports = self._plugin.get_ports(context, filters=dict(
             network_id=[network_id], device_owner=[const.DEVICE_OWNER_DHCP]))
-        # There should be only one metadata port per network
-        if len(ports) == 1:
-            return ports[0]
+
+        # Metadata ports are DHCP ports not belonging to the Neutron
+        # DHCP agents
+        for port in ports:
+            if not utils.is_neutron_dhcp_agent_port(port):
+                return port
 
     def _find_metadata_port_ip(self, context, subnet):
         metadata_port = self._find_metadata_port(context, subnet['network_id'])
@@ -1787,7 +1857,8 @@ class OVNClient(object):
                 port = {'port':
                         {'network_id': network['id'],
                          'tenant_id': network['project_id'],
-                         'device_owner': const.DEVICE_OWNER_DHCP}}
+                         'device_owner': const.DEVICE_OWNER_DHCP,
+                         'device_id': 'ovnmeta-%s' % network['id']}}
                 # TODO(boden): rehome create_port into neutron-lib
                 p_utils.create_port(self._plugin, context, port)
             elif len(metadata_ports) > 1:
