@@ -10,6 +10,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import inspect
 import os
 import re
 
@@ -311,11 +312,10 @@ def get_lrouter_ext_gw_static_route(ovn_router):
     # TODO(lucasagomes): Remove the try...except block after OVS 2.8.2
     # is tagged.
     try:
-        for route in getattr(ovn_router, 'static_routes', []):
-            external_ids = getattr(route, 'external_ids', {})
-            if strutils.bool_from_string(
-                external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
-                return route
+        return [route for route in getattr(ovn_router, 'static_routes', []) if
+                strutils.bool_from_string(getattr(
+                    route, 'external_ids', {}).get(
+                        constants.OVN_ROUTER_IS_EXT_GW, 'false'))]
     except KeyError:
         pass
 
@@ -366,3 +366,51 @@ def get_system_dns_resolvers(resolver_file=DNS_RESOLVER_FILE):
 def get_port_subnet_ids(port):
     fixed_ips = [ip for ip in port['fixed_ips']]
     return [f['subnet_id'] for f in fixed_ips]
+
+
+def ovn_metadata_name(id_):
+    """Return the OVN metadata name based on an id."""
+    return 'metadata-%s' % id_
+
+
+def get_method_class(method):
+    if not inspect.ismethod(method):
+        return
+    return method.__self__.__class__
+
+
+def is_gateway_chassis_invalid(chassis_name, gw_chassis,
+                               physnet, chassis_physnets):
+    """Check if gateway chassis is invalid
+
+    @param    chassis_name: gateway chassis name
+    @type     chassis_name: string
+    @param    gw_chassis: List of gateway chassis in the system
+    @type     gw_chassis: []
+    @param    physnet: physical network associated to chassis_name
+    @type     physnet: string
+    @param    chassis_physnets: Dictionary linking chassis with their physnets
+    @type     chassis_physnets: {}
+    @return   Boolean
+    """
+
+    if chassis_name == constants.OVN_GATEWAY_INVALID_CHASSIS:
+        return True
+    elif chassis_name not in chassis_physnets:
+        return True
+    elif physnet and physnet not in chassis_physnets.get(chassis_name):
+        return True
+    elif gw_chassis and chassis_name not in gw_chassis:
+        return True
+    return False
+
+
+def is_neutron_dhcp_agent_port(port):
+    """Check if the given DHCP port belongs to Neutron DHCP agents
+
+    The DHCP ports with the device_id equals to 'reserved_dhcp_port'
+    or starting with the word 'dhcp' belongs to the Neutron DHCP agents.
+    """
+    return (port['device_owner'] == const.DEVICE_OWNER_DHCP and
+            (port['device_id'] == const.DEVICE_ID_RESERVED_DHCP_PORT or
+             port['device_id'].startswith('dhcp')))
