@@ -12,7 +12,7 @@
 #    under the License.
 #
 
-import datetime
+import copy
 import functools
 import operator
 import threading
@@ -32,15 +32,19 @@ from neutron_lib.services.qos import constants as qos_consts
 from oslo_config import cfg
 from oslo_db import exception as os_db_exc
 from oslo_log import log
+from oslo_utils import timeutils
 
 from neutron.common import utils as n_utils
 from neutron.db import provisioning_blocks
+from neutron.plugins.ml2 import db as ml2_db
 from neutron.services.segments import db as segment_service_db
 
 from networking_ovn._i18n import _
+from networking_ovn.agent import stats
 from networking_ovn.common import acl as ovn_acl
 from networking_ovn.common import config
 from networking_ovn.common import constants as ovn_const
+from networking_ovn.common import exceptions as ovn_exc
 from networking_ovn.common import maintenance
 from networking_ovn.common import ovn_client
 from networking_ovn.common import utils
@@ -136,7 +140,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             portbindings.VIF_TYPE_VHOST_USER: {
                 portbindings.CAP_PORT_FILTER: False,
                 portbindings.VHOST_USER_MODE:
-                portbindings.VHOST_USER_MODE_CLIENT,
+                portbindings.VHOST_USER_MODE_SERVER,
                 portbindings.VHOST_USER_OVS_PLUG: True
             }
         }
@@ -190,7 +194,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         # Now IDL connections can be safely used.
         self._post_fork_event.set()
 
-        if trigger.im_class == ovsdb_monitor.OvnWorker:
+        if utils.get_method_class(trigger) == ovsdb_monitor.OvnWorker:
             # Call the synchronization task if its ovn worker
             # This sync neutron DB to OVN-NB DB only in inconsistent states
             self.nb_synchronizer = ovn_db_sync.OvnNbSynchronizer(
@@ -210,7 +214,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             )
             self.sb_synchronizer.sync()
 
-        if trigger.im_class == maintenance.MaintenanceWorker:
+        if utils.get_method_class(trigger) == maintenance.MaintenanceWorker:
             self._maintenance_thread = maintenance.MaintenanceThread()
             self._maintenance_thread.add_periodics(
                 maintenance.DBInconsistenciesPeriodics(self._ovn_client))
@@ -392,7 +396,8 @@ class OVNMechanismDriver(api.MechanismDriver):
             return
         utils.validate_and_get_data_from_binding_profile(port)
         if self._is_port_provisioning_required(port, context.host):
-            self._insert_port_provisioning_block(context._plugin_context, port)
+            self._insert_port_provisioning_block(context._plugin_context,
+                                                 port['id'])
 
         db_rev.create_initial_revision(port['id'],
                                        ovn_const.TYPE_PORTS,
@@ -436,13 +441,12 @@ class OVNMechanismDriver(api.MechanismDriver):
 
         return True
 
-    def _insert_port_provisioning_block(self, context, port):
+    def _insert_port_provisioning_block(self, context, port_id):
         # Insert a provisioning block to prevent the port from
         # transitioning to active until OVN reports back that
         # the port is up.
         provisioning_blocks.add_provisioning_component(
-            context,
-            port['id'], resources.PORT,
+            context, port_id, resources.PORT,
             provisioning_blocks.L2_AGENT_ENTITY
         )
 
@@ -481,7 +485,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         drastically affect performance.  Raising an exception will
         result in the deletion of the resource.
         """
-        port = context.current
+        port = copy.deepcopy(context.current)
         port['network'] = context.network.current
         self._ovn_client.create_port(port)
         self._notify_dhcp_updated(port['id'])
@@ -507,7 +511,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         utils.validate_and_get_data_from_binding_profile(port)
         if self._is_port_provisioning_required(port, context.host,
                                                context.original_host):
-            self._insert_port_provisioning_block(context._plugin_context, port)
+            self._insert_port_provisioning_block(context._plugin_context,
+                                                 port['id'])
 
         if utils.is_lsp_router_port(port):
             # handle the case when an existing port is added to a
@@ -515,7 +520,8 @@ class OVNMechanismDriver(api.MechanismDriver):
             if not utils.is_lsp_router_port(original_port):
                 db_rev.create_initial_revision(port['id'],
                                                ovn_const.TYPE_ROUTER_PORTS,
-                                               context._plugin_context.session)
+                                               context._plugin_context.session,
+                                               may_exist=True)
 
     def update_port_postcommit(self, context):
         """Update a port.
@@ -533,10 +539,30 @@ class OVNMechanismDriver(api.MechanismDriver):
         state. It is up to the mechanism driver to ignore state or
         state changes that it does not know or care about.
         """
-        port = context.current
+        port = copy.deepcopy(context.current)
         port['network'] = context.network.current
-        original_port = context.original
+        original_port = copy.deepcopy(context.original)
         original_port['network'] = context.network.current
+
+        # NOTE(mjozefcz): Check if port is in migration state. If so update
+        # the port status from DOWN to UP in order to generate 'fake'
+        # vif-interface-plugged event. This workaround is needed to
+        # perform live-migration with live_migration_wait_for_vif_plug=True.
+        if ((port['status'] == const.PORT_STATUS_DOWN and
+             ovn_const.MIGRATING_ATTR in port[portbindings.PROFILE].keys() and
+             port[portbindings.VIF_TYPE] == portbindings.VIF_TYPE_OVS)):
+            admin_context = n_context.get_admin_context()
+            LOG.info("Setting port %s status from DOWN to UP in order "
+                     "to emit vif-interface-plugged event.",
+                     port['id'])
+            self._plugin.update_port_status(admin_context, port['id'],
+                                            const.PORT_STATUS_ACTIVE)
+            # The revision has been changed. In the meantime
+            # port-update event already updated the OVN configuration,
+            # So there is no need to update it again here. Anyway it
+            # will fail that OVN has port with bigger revision.
+            return
+
         self._ovn_client.update_port(port, port_object=original_port)
         self._notify_dhcp_updated(port['id'])
 
@@ -552,7 +578,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         expected, and will not prevent the resource from being
         deleted.
         """
-        port = context.current
+        port = copy.deepcopy(context.current)
         port['network'] = context.network.current
         self._ovn_client.delete_port(port['id'], port_object=port)
 
@@ -693,19 +719,6 @@ class OVNMechanismDriver(api.MechanismDriver):
         # See doc/source/design/ovn_worker.rst for more details.
         return [ovsdb_monitor.OvnWorker(), maintenance.MaintenanceWorker()]
 
-    def _update_subport_host_if_needed(self, port_id):
-        parent_port = self._ovn_client.get_parent_port(port_id)
-        if parent_port:
-            admin_context = n_context.get_admin_context()
-            try:
-                port = self._plugin.get_port(admin_context, parent_port)
-                host_id = port.get(portbindings.HOST_ID, '')
-                subport = {'port': {'binding:host_id': host_id}}
-                self._plugin.update_port(admin_context, port_id, subport)
-            except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
-                LOG.debug("Error trying to set host_id %s for subport %s",
-                          host_id, port_id)
-
     def _update_dnat_entry_if_needed(self, port_id, up=True):
         """Update DNAT entry if using distributed floating ips."""
         if not config.is_ovn_distributed_floating_ip():
@@ -738,7 +751,7 @@ class OVNMechanismDriver(api.MechanismDriver):
                 'NAT', nat['_uuid'],
                 ('external_mac', mac)).execute(check_error=True)
         else:
-            LOG.debug("Clearing up external_mac of port %s", port_id, mac)
+            LOG.debug("Clearing up external_mac of port %s", port_id)
             self._nb_ovn.db_clear(
                 'NAT', nat['_uuid'], 'external_mac').execute(check_error=True)
 
@@ -751,11 +764,6 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._update_dnat_entry_if_needed(port_id)
         self._wait_for_metadata_provisioned_if_needed(port_id)
 
-        # If this port is a subport, we need to update the host_id and set it
-        # to its parent's. Otherwise, Neutron won't even try to bind it and
-        # it will not transition from DOWN to ACTIVE.
-        self._update_subport_host_if_needed(port_id)
-
         provisioning_blocks.provisioning_complete(
             n_context.get_admin_context(),
             port_id,
@@ -770,12 +778,18 @@ class OVNMechanismDriver(api.MechanismDriver):
             # becasue the router ports are unbind so, for OVN we are
             # forcing the status here. Maybe it's something that we can
             # change in core Neutron in the future.
-            port = self._plugin.get_port(admin_context, port_id)
-            if port.get('device_owner') in (const.DEVICE_OWNER_ROUTER_INTF,
-                                            const.DEVICE_OWNER_DVR_INTERFACE,
-                                            const.DEVICE_OWNER_ROUTER_HA_INTF):
+            db_port = ml2_db.get_port(admin_context, port_id)
+            if not db_port:
+                return
+
+            if db_port.device_owner in (const.DEVICE_OWNER_ROUTER_INTF,
+                                        const.DEVICE_OWNER_DVR_INTERFACE,
+                                        const.DEVICE_OWNER_ROUTER_HA_INTF):
                 self._plugin.update_port_status(admin_context, port_id,
                                                 const.PORT_STATUS_ACTIVE)
+            elif db_port.device_owner.startswith(
+                    const.DEVICE_OWNER_COMPUTE_PREFIX):
+                self._plugin.nova_notifier.notify_port_active_direct(db_port)
         except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
             LOG.debug('Port not found during OVN status up report: %s',
                       port_id)
@@ -790,15 +804,32 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._update_dnat_entry_if_needed(port_id, False)
         admin_context = n_context.get_admin_context()
         try:
-            port = self._plugin.get_port(admin_context, port_id)
-            self._insert_port_provisioning_block(admin_context, port)
-            self._plugin.update_port_status(admin_context,
-                                            port['id'],
+            db_port = ml2_db.get_port(admin_context, port_id)
+            if not db_port:
+                return
+
+            self._insert_port_provisioning_block(admin_context, port_id)
+            self._plugin.update_port_status(admin_context, port_id,
                                             const.PORT_STATUS_DOWN)
+
+            if db_port.device_owner.startswith(
+                    const.DEVICE_OWNER_COMPUTE_PREFIX):
+                self._plugin.nova_notifier.record_port_status_changed(
+                    db_port, const.PORT_STATUS_ACTIVE, const.PORT_STATUS_DOWN,
+                    None)
+                self._plugin.nova_notifier.send_port_status(
+                    None, None, db_port)
         except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
             LOG.debug("Port not found during OVN status down report: %s",
                       port_id)
 
+    def delete_mac_binding_entries(self, external_ip):
+        """Delete all MAC_Binding entries associated to this IP address"""
+        mac_binds = self._sb_ovn.db_find_rows(
+            'MAC_Binding', ('ip', '=', external_ip)).execute() or []
+        for entry in mac_binds:
+            self._sb_ovn.db_destroy('MAC_Binding', entry.uuid).execute()
+
     def update_segment_host_mapping(self, host, phy_nets):
         """Update SegmentHostMapping in DB"""
         if not host:
@@ -830,8 +861,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         """Wait for metadata service to be provisioned.
 
         Wait until metadata service has been setup for this port in the chassis
-        it resides. If metadata is disabled, this function will return right
-        away.
+        it resides. If metadata is disabled or DHCP is not enabled for its
+        subnets, this function will return right away.
         """
         if config.is_ovn_metadata_enabled() and self._sb_ovn:
             # Wait until metadata service has been setup for this port in the
@@ -846,6 +877,26 @@ class OVNMechanismDriver(api.MechanismDriver):
                 LOG.warning("Logical port %s is not bound to a "
                             "chassis", port_id)
                 return
+
+            # Check if the port belongs to some IPv4 subnet with DHCP enabled.
+            context = n_context.get_admin_context()
+            port = self._plugin.get_port(context, port_id)
+            port_subnet_ids = set(
+                ip['subnet_id'] for ip in port['fixed_ips'] if
+                n_utils.get_ip_version(ip['ip_address']) == const.IP_VERSION_4)
+            if not port_subnet_ids:
+                # The port doesn't belong to any IPv4 subnet
+                return
+
+            subnets = self._plugin.get_subnets(context, filters=dict(
+                network_id=[port['network_id']], ip_version=[4],
+                enable_dhcp=True))
+
+            subnet_ids = set(
+                s['id'] for s in subnets if s['id'] in port_subnet_ids)
+            if not subnet_ids:
+                return
+
             try:
                 n_utils.wait_until_true(
                     lambda: datapath in
@@ -861,30 +912,77 @@ class OVNMechanismDriver(api.MechanismDriver):
                             " networking-ovn-metadata-agent status/logs.",
                             port_id)
 
-    def agent_from_chassis(self, chassis):
-        agent_type = ovn_const.OVN_CONTROLLER_AGENT
-        if ('enable-chassis-as-gw' in
-                chassis.external_ids.get('ovn-cms-options', [])):
-            agent_type = ovn_const.OVN_CONTROLLER_GW_AGENT
-        alive = self._nb_ovn.nb_global.nb_cfg == chassis.nb_cfg
+    def agent_alive(self, chassis, type_):
+        nb_cfg = chassis.nb_cfg
+        id_ = chassis.uuid
+        if type_ == ovn_const.OVN_METADATA_AGENT:
+            id_ = utils.ovn_metadata_name(chassis.uuid)
+            nb_cfg = int(chassis.external_ids.get(
+                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY, 0))
+
+        if self._nb_ovn.nb_global.nb_cfg == nb_cfg:
+            return True
+        now = timeutils.utcnow()
+
+        try:
+            updated_at = stats.AgentStats.get_stat(id_).updated_at
+        except ovn_exc.AgentStatsNotFound:
+            return False
+
+        if (now - updated_at).total_seconds() < cfg.CONF.agent_down_time:
+            return True
+        return False
+
+    def _format_agent_info(self, chassis, binary, agent_id, type_,
+                           description, alive):
         return {
-            'binary': "ovn-controller",
+            'binary': binary,
             'host': chassis.hostname,
-            'heartbeat_timestamp': datetime.datetime.utcnow(),
+            'heartbeat_timestamp': timeutils.utcnow(),
             'availability_zone': 'n/a',
             'topic': 'n/a',
-            'description':
-                chassis.external_ids.get(ovn_const.OVN_AGENT_DESC_KEY, ''),
+            'description': description,
             'configurations': {
                 'chassis_name': chassis.name,
                 'bridge-mappings':
                     chassis.external_ids.get('ovn-bridge-mappings', '')},
             'start_flag': True,
-            'agent_type': agent_type,
-            'id': str(chassis.uuid),
+            'agent_type': type_,
+            'id': agent_id,
             'alive': alive,
             'admin_state_up': True}
 
+    def agents_from_chassis(self, chassis):
+        agent_dict = {}
+
+        # Check for ovn-controller / ovn-controller gateway
+        agent_type = ovn_const.OVN_CONTROLLER_AGENT
+        agent_id = str(chassis.uuid)
+        if ('enable-chassis-as-gw' in
+                chassis.external_ids.get('ovn-cms-options', [])):
+            agent_type = ovn_const.OVN_CONTROLLER_GW_AGENT
+
+        alive = self.agent_alive(chassis, agent_type)
+        description = chassis.external_ids.get(
+            ovn_const.OVN_AGENT_DESC_KEY, '')
+        agent_dict[agent_id] = self._format_agent_info(
+            chassis, 'ovn-controller', agent_id, agent_type, description,
+            alive)
+
+        # Check for the metadata agent
+        metadata_agent_id = chassis.external_ids.get(
+            ovn_const.OVN_AGENT_METADATA_ID_KEY)
+        if metadata_agent_id:
+            agent_type = ovn_const.OVN_METADATA_AGENT
+            alive = self.agent_alive(chassis, agent_type)
+            description = chassis.external_ids.get(
+                ovn_const.OVN_AGENT_METADATA_DESC_KEY, '')
+            agent_dict[metadata_agent_id] = self._format_agent_info(
+                chassis, 'networking-ovn-metadata-agent',
+                metadata_agent_id, agent_type, description, alive)
+
+        return agent_dict
+
     def patch_plugin_merge(self, method_name, new_fn, op=operator.add):
         old_method = getattr(self._plugin, method_name)
 
@@ -909,24 +1007,46 @@ class OVNMechanismDriver(api.MechanismDriver):
 
         setattr(self._plugin, method_name, types.MethodType(fn, self._plugin))
 
+    def ping_chassis(self):
+        """Update NB_Global.nb_cfg so that Chassis.nb_cfg will increment"""
+
+        with self._nb_ovn.create_transaction(check_error=True,
+                                             bump_nb_cfg=True) as txn:
+            txn.add(self._nb_ovn.check_liveness())
+
 
 def get_agents(self, context, filters=None, fields=None, _driver=None):
+    _driver.ping_chassis()
     filters = filters or {}
-    agents = []
-    for c in _driver._sb_ovn.tables['Chassis'].rows.values():
-        a = _driver.agent_from_chassis(c)
-        if all(a[k] in v for k, v in filters.items()):
-            agents.append(a)
-    return agents
+    agent_list = []
+    for ch in _driver._sb_ovn.tables['Chassis'].rows.values():
+        for agent in _driver.agents_from_chassis(ch).values():
+            if all(agent[k] in v for k, v in filters.items()):
+                agent_list.append(agent)
+    return agent_list
 
 
 def get_agent(self, context, id, fields=None, _driver=None):
-    c = _driver._sb_ovn.tables['Chassis'].rows[uuid.UUID(id)]
-    return _driver.agent_from_chassis(c)
+    chassis = None
+    try:
+        chassis = _driver._sb_ovn.tables['Chassis'].rows[uuid.UUID(id)]
+    except KeyError:
+        # If the UUID is not found, check for the metadata agent ID
+        for ch in _driver._sb_ovn.tables['Chassis'].rows.values():
+            metadata_agent_id = ch.external_ids.get(
+                ovn_const.OVN_AGENT_METADATA_ID_KEY)
+            if id == metadata_agent_id:
+                chassis = ch
+                break
+        else:
+            raise
+    return _driver.agents_from_chassis(chassis)[id]
 
 
 def update_agent(self, context, id, agent, _driver=None):
-    chassis = _driver._sb_ovn.tables['Chassis'].rows[uuid.UUID(id)]
+    ovn_agent = get_agent(self, None, id, _driver=_driver)
+    chassis_name = ovn_agent['configurations']['chassis_name']
+    agent_type = ovn_agent['agent_type']
     agent = agent['agent']
     # neutron-client always passes admin_state_up, openstack client doesn't
     # and we can just fall through to raising in the case that admin_state_up
@@ -935,16 +1055,17 @@ def update_agent(self, context, id, agent, _driver=None):
         pass
     elif 'description' in agent:
         _driver._sb_ovn.set_chassis_neutron_description(
-            chassis.name, agent['description']).execute(check_error=True)
-        return _driver.agent_from_chassis(chassis)
+            chassis_name, agent['description'],
+            agent_type).execute(check_error=True)
+        return agent
     else:
         # admin_state_up=True w/o description
-        return _driver.agent_from_chassis(chassis)
+        return agent
     raise n_exc.BadRequest(resource='agent',
                            msg='OVN agent status cannot be updated')
 
 
 def delete_agent(self, context, id, _driver=None):
-    c = _driver._sb_ovn.tables['Chassis'].rows[uuid.UUID(id)]  # noqa
+    get_agent(self, None, id, _driver=_driver)
     raise n_exc.BadRequest(resource='agent',
                            msg='OVN agents cannot be deleted')
