@@ -40,7 +40,16 @@ LOG = log.getLogger(__name__)
 
 
 class OvnNbTransaction(idl_trans.Transaction):
+
+    def __init__(self, *args, **kwargs):
+        # NOTE(lucasagomes): The bump_nb_cfg parameter is only used by
+        # the agents health status check
+        self.bump_nb_cfg = kwargs.pop('bump_nb_cfg', False)
+        super(OvnNbTransaction, self).__init__(*args, **kwargs)
+
     def pre_commit(self, txn):
+        if not self.bump_nb_cfg:
+            return
         self.api.nb_global.increment('nb_cfg')
 
 
@@ -117,8 +126,9 @@ def get_ovn_idls(driver, trigger):
         wait=tenacity.wait_exponential(max=180),
         reraise=True)
     def get_ovn_idl_retry(cls):
+        trigger_class = utils.get_method_class(trigger)
         LOG.info('Getting %(cls)s for %(trigger)s with retry',
-                 {'cls': cls.__name__, 'trigger': trigger.im_class.__name__})
+                 {'cls': cls.__name__, 'trigger': trigger_class.__name__})
         return cls(get_connection(cls, trigger, driver))
 
     vlog.use_python_logger(max_level=cfg.get_ovn_ovsdb_log_level())
@@ -134,7 +144,7 @@ def get_connection(db_class, trigger=None, driver=None):
         args = (cfg.get_ovn_sb_connection(), 'OVN_Southbound')
         cls = ovsdb_monitor.OvnSbIdl
 
-    if trigger and trigger.im_class == ovsdb_monitor.OvnWorker:
+    if trigger and utils.get_method_class(trigger) == ovsdb_monitor.OvnWorker:
         idl_ = cls.from_server(*args, driver=driver)
     else:
         if db_class == OvsdbSbOvnIdl:
@@ -154,10 +164,11 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def nb_global(self):
         return next(iter(self.tables['NB_Global'].rows.values()))
 
-    def create_transaction(self, check_error=False, log_errors=True):
+    def create_transaction(self, check_error=False, log_errors=True,
+                           bump_nb_cfg=False):
         return OvnNbTransaction(
             self, self.ovsdb_connection, self.ovsdb_connection.timeout,
-            check_error, log_errors)
+            check_error, log_errors, bump_nb_cfg=bump_nb_cfg)
 
     @contextlib.contextmanager
     def transaction(self, *args, **kwargs):
@@ -383,7 +394,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
         @param   lrp: logical router port
         @type    lrp: Logical_Router_Port row
-        @return: List of tuples (chassis_name, priority)
+        @return: List of tuples (chassis_name, priority) sorted by priority
         """
         # Try retrieving gateway_chassis with new schema. If new schema is not
         # supported or user is using old schema, then use old schema for
@@ -396,7 +407,8 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
             rc = lrp.options.get(ovn_const.OVN_GATEWAY_CHASSIS_KEY)
             if rc:
                 chassis.append((rc, 0))
-        return chassis
+        # make sure that chassis are sorted by priority
+        return sorted(chassis, reverse=True, key=lambda x: x[1])
 
     def get_all_chassis_gateway_bindings(self,
                                          chassis_candidate_list=None):
@@ -427,21 +439,18 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_unhosted_gateways(self, port_physnet_dict, chassis_physnets,
                               gw_chassis):
         unhosted_gateways = []
-        valid_chassis_list = list(chassis_physnets)
         for lrp in self._tables['Logical_Router_Port'].rows.values():
             if not lrp.name.startswith('lrp-'):
                 continue
             physnet = port_physnet_dict.get(lrp.name[len('lrp-'):])
             chassis_list = self._get_logical_router_port_gateway_chassis(lrp)
+            is_max_gw_reached = len(chassis_list) < ovn_const.MAX_GW_CHASSIS
             for chassis_name, prio in chassis_list:
                 # TODO(azbiswas): Handle the case when a chassis is no
                 # longer valid. This may involve moving conntrack states,
                 # so it needs to discussed in the OVN community first.
-                if (chassis_name == ovn_const.OVN_GATEWAY_INVALID_CHASSIS or
-                        chassis_name not in valid_chassis_list or
-                        (physnet and
-                         physnet not in chassis_physnets.get(chassis_name)) or
-                        (gw_chassis and chassis_name not in gw_chassis)):
+                if is_max_gw_reached or utils.is_gateway_chassis_invalid(
+                        chassis_name, gw_chassis, physnet, chassis_physnets):
                     unhosted_gateways.append(lrp.name)
         return unhosted_gateways
 
@@ -671,10 +680,39 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_port_group(self, pg_name):
         if uuidutils.is_uuid_like(pg_name):
             pg_name = utils.ovn_port_group_name(pg_name)
+        try:
+            for pg in self._tables['Port_Group'].rows.values():
+                if pg.name == pg_name:
+                    return pg
+        except KeyError:
+            # TODO(dalvarez): This except block is added for backwards compat
+            # with old OVN schemas (<=2.9) where Port Groups are not present.
+            # This (and other conditional code around this feature) shall be
+            # removed at some point.
+            return
 
-        for pg in self._tables['Port_Group'].rows.values():
-            if pg.name == pg_name:
-                return pg
+    def get_port_groups(self):
+        port_groups = {}
+        try:
+            for row in self._tables['Port_Group'].rows.values():
+                name = getattr(row, 'name')
+                if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
+                   name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
+                    continue
+                data = {}
+                for row_key in getattr(row, "_data", {}):
+                    data[row_key] = getattr(row, row_key)
+                port_groups[name] = data
+        except KeyError:
+            # TODO(dalvarez): This except block is added for backwards compat
+            # with old OVN schemas (<=2.9) where Port Groups are not present.
+            # This (and other conditional code around this feature) shall be
+            # removed at some point.
+            pass
+        return port_groups
+
+    def check_liveness(self):
+        return cmd.CheckLivenessCommand(self)
 
 
 class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
@@ -755,10 +793,13 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             self, chassis, {'neutron-metadata-proxy-networks': nets},
             if_exists=True)
 
-    def set_chassis_neutron_description(self, chassis, description):
+    def set_chassis_neutron_description(self, chassis, description,
+                                        agent_type):
+        desc_key = (ovn_const.OVN_AGENT_METADATA_DESC_KEY
+                    if agent_type == ovn_const.OVN_METADATA_AGENT else
+                    ovn_const.OVN_AGENT_DESC_KEY)
         return cmd.UpdateChassisExtIdsCommand(
-            self, chassis, {ovn_const.OVN_AGENT_DESC_KEY: description},
-            if_exists=False)
+            self, chassis, {desc_key: description}, if_exists=False)
 
     def get_network_port_bindings_by_ip(self, network, ip_address):
         rows = self.db_list_rows('Port_Binding').execute(check_error=True)
@@ -768,6 +809,12 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
                 if (r.mac and str(r.datapath.uuid) == network) and
                 ip_address in r.mac[0].split(' ')]
 
+    def update_metadata_health_status(self, chassis, nb_cfg):
+        return cmd.UpdateChassisExtIdsCommand(
+            self, chassis,
+            {ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: str(nb_cfg)},
+            if_exists=True)
+
     def set_port_cidrs(self, name, cidrs):
         # TODO(twilson) add if_exists to db commands
         return self.db_set('Port_Binding', name, 'external_ids',
