@@ -12,6 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import abc
+
 from neutron.common import config
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
@@ -24,12 +26,83 @@ from ovsdbapp.backend.ovs_idl import event as row_event
 from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import event
 
+from networking_ovn.agent import stats
 from networking_ovn.common import config as ovn_config
+from networking_ovn.common import constants as ovn_const
 from networking_ovn.common import utils
 
 LOG = log.getLogger(__name__)
 
 
+class BaseEvent(row_event.RowEvent):
+    table = None
+    events = tuple()
+
+    def __init__(self):
+        self.event_name = self.__class__.__name__
+        super(BaseEvent, self).__init__(self.events, self.table, None)
+
+    @abc.abstractmethod
+    def match_fn(self, event, row, old=None):
+        """Define match criteria other than table/event"""
+
+    def matches(self, event, row, old=None):
+        if row._table.name != self.table or event not in self.events:
+            return False
+        if not self.match_fn(event, row, old):
+            return False
+        LOG.debug("%s : Matched %s, %s, %s %s", self.event_name, self.table,
+                  event, self.conditions, self.old_conditions)
+        return True
+
+
+class ChassisAgentDeleteEvent(BaseEvent):
+    table = 'Chassis'
+    events = (BaseEvent.ROW_DELETE,)
+
+    def run(self, event, row, old):
+        stats.AgentStats.del_agent(row.uuid)
+        stats.AgentStats.del_agent(utils.ovn_metadata_name(row.uuid))
+
+    def match_fn(self, event, row, old=None):
+        return True
+
+
+class ChassisGatewayAgentEvent(BaseEvent):
+    table = 'Chassis'
+    events = (BaseEvent.ROW_CREATE, BaseEvent.ROW_UPDATE)
+
+    def match_fn(self, event, row, old=None):
+        return event == self.ROW_CREATE or getattr(old, 'nb_cfg', False)
+
+    def run(self, event, row, old):
+        stats.AgentStats.add_stat(row.uuid, row.nb_cfg)
+
+
+class ChassisMetadataAgentEvent(BaseEvent):
+    table = 'Chassis'
+    events = (BaseEvent.ROW_CREATE, BaseEvent.ROW_UPDATE)
+
+    @staticmethod
+    def _metadata_nb_cfg(row):
+        return int(row.external_ids[ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY])
+
+    def match_fn(self, event, row, old=None):
+        if event == self.ROW_CREATE:
+            return True
+        try:
+            return self._metadata_nb_cfg(row) != self._metadata_nb_cfg(old)
+        except (AttributeError, KeyError):
+            return False
+
+    def run(self, event, row, old):
+        try:
+            stats.AgentStats.add_stat(utils.ovn_metadata_name(row.uuid),
+                                      self._metadata_nb_cfg(row))
+        except (AttributeError, KeyError):
+            return
+
+
 class ChassisEvent(row_event.RowEvent):
     """Chassis create update delete event."""
 
@@ -55,6 +128,46 @@ class ChassisEvent(row_event.RowEvent):
             self.l3_plugin.schedule_unhosted_gateways()
 
 
+class PortBindingChassisUpdateEvent(BaseEvent):
+    """Event for matching a port moving chassis
+
+    If the LSP is up and the Port_Binding chassis has just changed,
+    there is a good chance the host died without cleaning up the chassis
+    column on the Port_Binding. The port never goes down, so we won't
+    see update the driver with the LogicalSwitchPortUpdateUpEvent which
+    only monitors for transitions from DOWN to UP.
+    """
+
+    table = 'Port_Binding'
+    events = (BaseEvent.ROW_UPDATE,)
+
+    def __init__(self, driver):
+        self.driver = driver
+        super(PortBindingChassisUpdateEvent, self).__init__()
+
+    def match_fn(self, event, row, old=None):
+        # NOTE(twilson) ROW_UPDATE events always pass old, but chassis will
+        # only be set if chassis has changed
+        old_chassis = getattr(old, 'chassis', None)
+        if not (row.chassis and old_chassis) or row.chassis == old_chassis:
+            return False
+        if row.type == 'chassisredirect':
+            return False
+        try:
+            lsp = self.driver._nb_ovn.lookup('Logical_Switch_Port',
+                                             row.logical_port)
+        except idlutils.RowNotFound:
+            LOG.warning("Logical Switch Port %(port)s not found for "
+                        "Port_Binding %(binding)s",
+                        {'port': row.logical_port, 'binding': row.uuid})
+            return False
+
+        return bool(lsp.up)
+
+    def run(self, event, row, old=None):
+        self.driver.set_port_status_up(row.logical_port)
+
+
 class PortBindingChassisEvent(row_event.RowEvent):
     """Port_Binding update event - set chassis for chassisredirect port.
 
@@ -167,6 +280,27 @@ class LogicalSwitchPortUpdateDownEvent(row_event.RowEvent):
         self.driver.set_port_status_down(row.name)
 
 
+class FIPAddDeleteEvent(row_event.RowEvent):
+    """Row event - NAT 'dnat_and_snat' entry added or deleted
+
+    This happens when a FIP is created or removed.
+    """
+    def __init__(self, driver):
+        self.driver = driver
+        table = 'NAT'
+        events = (self.ROW_CREATE, self.ROW_DELETE)
+        super(FIPAddDeleteEvent, self).__init__(
+            events, table, (('type', '=', 'dnat_and_snat'),))
+        self.event_name = 'FIPAddDeleteEvent'
+
+    def run(self, event, row, old):
+        # When a FIP is added or deleted, we will delete all entries in the
+        # MAC_Binding table of SB OVSDB corresponding to that IP Address.
+        # TODO(dalvarez): Remove this workaround once fixed in core OVN:
+        # https://mail.openvswitch.org/pipermail/ovs-discuss/2018-October/047604.html
+        self.driver.delete_mac_binding_entries(row.external_ip)
+
+
 class OvnDbNotifyHandler(event.RowEventHandler):
     def __init__(self, driver):
         super(OvnDbNotifyHandler, self).__init__()
@@ -183,6 +317,13 @@ class BaseOvnIdl(connection.OvsdbIdl):
 
 
 class BaseOvnSbIdl(connection.OvsdbIdl):
+    def __init__(self, remote, schema):
+        super(BaseOvnSbIdl, self).__init__(remote, schema)
+        self.notify_handler = event.RowEventHandler()
+        self.notify_handler.watch_events([
+            ChassisAgentDeleteEvent(), ChassisMetadataAgentEvent(),
+            ChassisGatewayAgentEvent()])
+
     @classmethod
     def from_server(cls, connection_string, schema_name):
         _check_and_set_ssl_files(schema_name)
@@ -193,6 +334,9 @@ class BaseOvnSbIdl(connection.OvsdbIdl):
         helper.register_table('Datapath_Binding')
         return cls(connection_string, helper)
 
+    def notify(self, event, row, updates=None):
+        self.notify_handler.notify(event, row, updates)
+
 
 class OvnIdl(BaseOvnIdl):
 
@@ -239,11 +383,13 @@ class OvnNbIdl(OvnIdl):
         self._lsp_update_down_event = LogicalSwitchPortUpdateDownEvent(driver)
         self._lsp_create_up_event = LogicalSwitchPortCreateUpEvent(driver)
         self._lsp_create_down_event = LogicalSwitchPortCreateDownEvent(driver)
+        self._fip_create_delete_event = FIPAddDeleteEvent(driver)
 
         self.notify_handler.watch_events([self._lsp_create_up_event,
                                           self._lsp_create_down_event,
                                           self._lsp_update_up_event,
-                                          self._lsp_update_down_event])
+                                          self._lsp_update_down_event,
+                                          self._fip_create_delete_event])
 
     @classmethod
     def from_server(cls, connection_string, schema_name, driver):
@@ -283,6 +429,7 @@ class OvnSbIdl(OvnIdl):
         helper.register_table('Encap')
         helper.register_table('Port_Binding')
         helper.register_table('Datapath_Binding')
+        helper.register_table('MAC_Binding')
         _idl = cls(driver, connection_string, helper)
         _idl.set_lock(_idl.event_lock_name)
         return _idl
@@ -297,8 +444,9 @@ class OvnSbIdl(OvnIdl):
         """
         self._chassis_event = ChassisEvent(self.driver)
         self._portbinding_event = PortBindingChassisEvent(self.driver)
-        self.notify_handler.watch_events([self._chassis_event,
-                                          self._portbinding_event])
+        self.notify_handler.watch_events(
+            [self._chassis_event, self._portbinding_event,
+             PortBindingChassisUpdateEvent(self.driver)])
 
 
 def _check_and_set_ssl_files(schema_name):
