@@ -7,10 +7,11 @@ VENV=${1:-"dsvm-functional"}
 GATE_DEST=$BASE/new
 NEUTRON_PATH=$GATE_DEST/neutron
 DEVSTACK_PATH=$GATE_DEST/devstack
+NETWORKING_OVN_PATH=$GATE_DEST/networking-ovn
 GATE_STACK_USER=stack
 
 case $VENV in
-"dsvm-functional"|"dsvm-functional-py35")
+"dsvm-functional"|"dsvm-functional-py27")
     source $DEVSTACK_PATH/functions
     source $NEUTRON_PATH/devstack/lib/ovs
 
@@ -31,11 +32,14 @@ case $VENV in
     remove_ovs_packages
     # compile_ovs expects "DEST" to be defined
     DEST=$GATE_DEST
-    OVS_BRANCH=branch-2.11
     compile_ovs True /usr/local /var
 
     # Make the workspace owned by GATE_STACK_USER
     sudo chown -R $GATE_STACK_USER:$GATE_STACK_USER $BASE
+
+    source $NETWORKING_OVN_PATH/tools/configure_for_func_testing.sh
+
+    configure_host_for_func_testing
     ;;
 
 *)
