@@ -20,15 +20,18 @@ import mock
 from neutron.conf.plugins.ml2 import config
 from neutron.plugins.ml2.drivers import type_geneve  # noqa
 from neutron.tests.unit.plugins.ml2 import test_plugin
+from neutron.tests.unit import testlib_api
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
+from oslo_concurrency import lockutils
 from oslo_config import cfg
+from oslo_db import exception as os_db_exc
+from oslo_db.sqlalchemy import provision
 from oslo_log import log
 from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import command
 from ovsdbapp.backend.ovs_idl import connection
 
-
 # Load all the models to register them into SQLAlchemy metadata before using
 # the SqlFixture
 from networking_ovn.db import models  # noqa
@@ -42,6 +45,7 @@ LOG = log.getLogger(__name__)
 # This is the directory from which infra fetches log files for functional tests
 DEFAULT_LOG_DIR = os.path.join(os.environ.get('OS_LOG_PATH', '/tmp'),
                                'dsvm-functional-logs')
+SQL_FIXTURE_LOCK = 'sql_fixture_lock'
 
 
 class AddFakeChassisCommand(command.BaseCommand):
@@ -80,6 +84,27 @@ class ConnectionFixture(fixtures.Fixture):
         self.connection.stop()
 
 
+class OVNSqlFixture(testlib_api.StaticSqlFixture):
+
+    @classmethod
+    @lockutils.synchronized(SQL_FIXTURE_LOCK)
+    def _init_resources(cls):
+        cls.schema_resource = provision.SchemaResource(
+            provision.DatabaseResource("sqlite"),
+            cls._generate_schema, teardown=False)
+        dependency_resources = {}
+        for name, resource in cls.schema_resource.resources:
+            dependency_resources[name] = resource.getResource()
+        cls.schema_resource.make(dependency_resources)
+        cls.engine = dependency_resources['database'].engine
+
+    def _delete_from_schema(self, engine):
+        try:
+            super(OVNSqlFixture, self)._delete_from_schema(engine)
+        except os_db_exc.DBNonExistentTable:
+            pass
+
+
 class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
 
     # Please see networking_ovn/tests/contrib/gate_hook.sh.
@@ -113,8 +138,20 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         self.mech_driver = mm.mech_drivers['ovn'].obj
         self.l3_plugin = directory.get_plugin(constants.L3)
         self.ovsdb_server_mgr = None
+        self.ovn_northd_mgr = None
         self.ovn_worker = ovn_worker
         self._start_ovsdb_server_and_idls()
+        self._start_ovn_northd()
+
+    # FIXME(lucasagomes): Workaround for
+    # https://bugs.launchpad.net/networking-ovn/+bug/1808146. We should
+    # investigate and properly fix the problem. This method is just a
+    # workaround to alleviate the gate for now and should not be considered
+    # a proper fix.
+    def _setup_database_fixtures(self):
+        fixture = OVNSqlFixture()
+        self.useFixture(fixture)
+        self.engine = fixture.engine
 
     def get_additional_service_plugins(self):
         p = super(TestOVNFunctionalBase, self).get_additional_service_plugins()
@@ -128,6 +165,16 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
     def get_ovsdb_server_protocol(self):
         return 'unix'
 
+    def _start_ovn_northd(self):
+        if not self.ovsdb_server_mgr:
+            return
+        ovn_nb_db = self.ovsdb_server_mgr.get_ovsdb_connection_path('nb')
+        ovn_sb_db = self.ovsdb_server_mgr.get_ovsdb_connection_path('sb')
+        self.ovn_northd_mgr = self.useFixture(
+            process.OvnNorthd(self.temp_dir,
+                              ovn_nb_db, ovn_sb_db,
+                              protocol=self._ovsdb_protocol))
+
     def _start_ovsdb_server_and_idls(self):
         self.temp_dir = self.useFixture(fixtures.TempDir()).path
         # Start 2 ovsdb-servers one each for OVN NB DB and OVN SB DB
@@ -194,16 +241,20 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
                 num_attempts += 1
                 time.sleep(1)
 
-        trigger = mock.MagicMock()
+        class TriggerCls(mock.MagicMock):
+            def trigger(self):
+                pass
+
+        trigger_cls = TriggerCls()
         if self.ovn_worker:
-            trigger.im_class = ovsdb_monitor.OvnWorker
+            trigger_cls.trigger.__self__.__class__ = ovsdb_monitor.OvnWorker
             cfg.CONF.set_override('neutron_sync_mode', 'off', 'ovn')
-        trigger.im_class.__name__ = 'trigger'
 
         self.addCleanup(self.stop)
 
         # mech_driver.post_fork_initialize creates the IDL connections
-        self.mech_driver.post_fork_initialize(mock.ANY, mock.ANY, trigger)
+        self.mech_driver.post_fork_initialize(
+            mock.ANY, mock.ANY, trigger_cls.trigger)
 
     def stop(self):
         if self.ovn_worker:
@@ -223,6 +274,8 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
 
         if self.ovsdb_server_mgr:
             self.ovsdb_server_mgr.stop()
+        if self.ovn_northd_mgr:
+            self.ovn_northd_mgr.stop()
 
         self.mech_driver._nb_ovn = None
         self.mech_driver._sb_ovn = None
@@ -232,14 +285,17 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
         self.sb_api.ovsdb_connection = None
 
         self._start_ovsdb_server_and_idls()
+        self._start_ovn_northd()
 
-    def add_fake_chassis(self, host, physical_nets=None, external_ids=None):
+    def add_fake_chassis(self, host, physical_nets=None, external_ids=None,
+                         name=None):
         physical_nets = physical_nets or []
         external_ids = external_ids or {}
 
         bridge_mapping = ",".join(["%s:br-provider%s" % (phys_net, i)
                                   for i, phys_net in enumerate(physical_nets)])
-        name = uuidutils.generate_uuid()
+        if name is None:
+            name = uuidutils.generate_uuid()
         external_ids['ovn-bridge-mappings'] = bridge_mapping
         # We'll be using different IP addresses every time for the Encap of
         # the fake chassis as the SB schema doesn't allow to have two entries
@@ -253,3 +309,7 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
             name, ['geneve'], '172.24.4.%d' % self._counter,
             external_ids=external_ids, hostname=host).execute(check_error=True)
         return name
+
+    def del_fake_chassis(self, chassis, if_exists=True):
+        self.sb_api.chassis_del(
+            chassis, if_exists=if_exists).execute(check_error=True)
