@@ -13,6 +13,7 @@
 #    under the License.
 
 from networking_ovn.common import utils
+from networking_ovn.db import revision as db_rev
 from networking_ovn.tests.functional import base
 from oslo_config import cfg
 from oslo_utils import uuidutils
@@ -25,6 +26,7 @@ class TestPortBinding(base.TestOVNFunctionalBase):
         self.ovs_host = 'ovs-host'
         self.dpdk_host = 'dpdk-host'
         self.invalid_dpdk_host = 'invalid-host'
+        self.vhu_mode = 'server'
         self.add_fake_chassis(self.ovs_host)
         self.add_fake_chassis(
             self.dpdk_host,
@@ -85,7 +87,7 @@ class TestPortBinding(base.TestOVNFunctionalBase):
 
         port_id = self._create_or_update_port(hostname=self.dpdk_host)
         expected_vif_details = {'port_filter': False,
-                                'vhostuser_mode': 'client',
+                                'vhostuser_mode': self.vhu_mode,
                                 'vhostuser_ovs_plug': True}
         expected_vif_details['vhostuser_socket'] = (
             utils.ovn_vhu_sockpath(cfg.CONF.ovn.vhost_sock_dir, port_id))
@@ -107,7 +109,7 @@ class TestPortBinding(base.TestOVNFunctionalBase):
         port_id = self._create_or_update_port(port_id=port_id,
                                               hostname=self.dpdk_host)
         expected_vif_details = {'port_filter': False,
-                                'vhostuser_mode': 'client',
+                                'vhostuser_mode': self.vhu_mode,
                                 'vhostuser_ovs_plug': True}
         expected_vif_details['vhostuser_socket'] = (
             utils.ovn_vhu_sockpath(cfg.CONF.ovn.vhost_sock_dir, port_id))
@@ -128,3 +130,47 @@ class TestPortBindingOverTcp(TestPortBinding):
 class TestPortBindingOverSsl(TestPortBinding):
     def get_ovsdb_server_protocol(self):
         return 'ssl'
+
+
+class TestNetworkMTUUpdate(base.TestOVNFunctionalBase):
+
+    def setUp(self):
+        super(TestNetworkMTUUpdate, self).setUp()
+        self._ovn_client = self.mech_driver._ovn_client
+        self.n1 = self._make_network(self.fmt, 'n1', True)
+        res = self._create_subnet(self.fmt, self.n1['network']['id'],
+                                  '10.0.0.0/24')
+        self.sub = self.deserialize(self.fmt, res)
+
+    def test_update_network_mtu(self):
+        mtu_value = self.n1['network']['mtu'] - 100
+        dhcp_options = (
+            self.mech_driver._ovn_client._nb_idl.get_subnet_dhcp_options(
+                self.sub['subnet']['id'])
+        )
+        self.assertNotEqual(
+            int(dhcp_options['subnet']['options']['mtu']),
+            mtu_value)
+        data = {'network': {'mtu': mtu_value}}
+        req = self.new_update_request(
+            'networks', data, self.n1['network']['id'], self.fmt)
+        req.get_response(self.api)
+        dhcp_options = (
+            self.mech_driver._ovn_client._nb_idl.get_subnet_dhcp_options(
+                self.sub['subnet']['id'])
+        )
+        self.assertEqual(
+            int(dhcp_options['subnet']['options']['mtu']),
+            mtu_value)
+
+    def test_no_update_network_mtu(self):
+        mtu_value = self.n1['network']['mtu']
+        base_revision = db_rev.get_revision_row(self.sub['subnet']['id'])
+        data = {'network': {'mtu': mtu_value}}
+        req = self.new_update_request(
+            'networks', data, self.n1['network']['id'], self.fmt)
+        req.get_response(self.api)
+        second_revision = db_rev.get_revision_row(self.sub['subnet']['id'])
+        self.assertEqual(
+            base_revision.updated_at,
+            second_revision.updated_at)
