@@ -63,22 +63,24 @@ class TestMetadataAgent(base.BaseTestCase):
         self.agent = agent.MetadataAgent(self.fake_conf)
         self.agent.sb_idl = mock.Mock()
         self.agent.ovs_idl = mock.Mock()
+        self.agent.ovs_idl.transaction = mock.MagicMock()
         self.agent.chassis = 'chassis'
+        self.agent.ovn_bridge = 'br-int'
 
     def test_sync(self):
         with mock.patch.object(
                 self.agent, 'ensure_all_networks_provisioned') as enp,\
                 mock.patch.object(
-                    ip_wrap, 'get_namespaces') as gns,\
+                    ip_lib, 'list_network_namespaces') as lnn,\
                 mock.patch.object(
                     self.agent, 'teardown_datapath') as tdp:
             enp.return_value = ['ovnmeta-1', 'ovnmeta-2']
-            gns.return_value = ['ovnmeta-1', 'ovnmeta-2']
+            lnn.return_value = ['ovnmeta-1', 'ovnmeta-2']
 
             self.agent.sync()
 
             enp.assert_called_once()
-            gns.assert_called_once()
+            lnn.assert_called_once()
             tdp.assert_not_called()
 
     def test_sync_teardown_namespace(self):
@@ -86,17 +88,17 @@ class TestMetadataAgent(base.BaseTestCase):
         with mock.patch.object(
                 self.agent, 'ensure_all_networks_provisioned') as enp,\
                 mock.patch.object(
-                    ip_wrap, 'get_namespaces') as gns,\
+                    ip_lib, 'list_network_namespaces') as lnn,\
                 mock.patch.object(
                     self.agent, 'teardown_datapath') as tdp:
             enp.return_value = ['ovnmeta-1', 'ovnmeta-2']
-            gns.return_value = ['ovnmeta-1', 'ovnmeta-2', 'ovnmeta-3',
+            lnn.return_value = ['ovnmeta-1', 'ovnmeta-2', 'ovnmeta-3',
                                 'ns1', 'ns2']
 
             self.agent.sync()
 
             enp.assert_called_once()
-            gns.assert_called_once()
+            lnn.assert_called_once()
             tdp.assert_called_once_with('3')
 
     def test_ensure_all_networks_provisioned(self):
@@ -181,8 +183,7 @@ class TestMetadataAgent(base.BaseTestCase):
             self.agent.teardown_datapath('1')
 
             destroy_mdp.assert_called_once()
-            self.agent.ovs_idl.del_port.assert_called_once_with(
-                'veth_0', bridge='br-int')
+            self.agent.ovs_idl.del_port.assert_called_once_with('veth_0')
             del_veth.assert_called_once_with('veth_0')
             garbage_collect.assert_called_once()
 
@@ -205,6 +206,8 @@ class TestMetadataAgent(base.BaseTestCase):
                                return_value=metadata_port),\
                 mock.patch.object(
                     ip_lib, 'device_exists', return_value=False),\
+                mock.patch.object(
+                    ip_lib.IPDevice, 'exists', return_value=False),\
                 mock.patch.object(agent.MetadataAgent, '_get_veth_name',
                                   return_value=['veth_0', 'veth_1']),\
                 mock.patch.object(agent.MetadataAgent, '_get_namespace_name',
@@ -224,8 +227,15 @@ class TestMetadataAgent(base.BaseTestCase):
                     driver.MetadataDriver,
                     'spawn_monitored_metadata_proxy') as spawn_mdp:
 
+            # Simulate that the VETH pair was already present in 'br-fake'.
+            # We need to assert that it was deleted first.
+            self.agent.ovs_idl.list_br.return_value.execute.return_value = (
+                ['br-int', 'br-fake'])
             self.agent.provision_datapath('1')
 
+            # Check that the port was deleted from br-fake
+            self.agent.ovs_idl.del_port.assert_called_once_with(
+                'veth_0', bridge='br-fake', if_exists=True)
             # Check that the VETH pair is created
             add_veth.assert_called_once_with('veth_0', 'veth_1', 'namespace')
             # Make sure that the two ends of the VETH pair have been set as up.
