@@ -102,6 +102,8 @@ class FakeOvsdbNbOvnIdl(object):
         self.get_lswitch = mock.Mock()
         fake_ovs_row = FakeOvsdbRow.create_one_ovsdb_row()
         self.get_lswitch.return_value = fake_ovs_row
+        self.get_lswitch_port = mock.Mock()
+        self.get_lswitch_port.return_value = fake_ovs_row
         self.get_ls_and_dns_record = mock.Mock()
         self.get_ls_and_dns_record.return_value = (fake_ovs_row, None)
         self.ls_set_dns_records = mock.Mock()
@@ -134,6 +136,13 @@ class FakeOvsdbNbOvnIdl(object):
         self.pg_add_ports = mock.Mock()
         self.pg_del_ports = mock.Mock()
         self.lsp_get_up = mock.Mock()
+        self.nb_global = mock.Mock()
+        self.db_list_rows = mock.Mock()
+        self.lsp_list = mock.MagicMock()
+        self.db_find = mock.Mock()
+        self.db_set = mock.Mock()
+        self.db_clear = mock.Mock()
+        self.db_remove = mock.Mock()
 
 
 class FakeOvsdbSbOvnIdl(object):
@@ -147,6 +156,11 @@ class FakeOvsdbSbOvnIdl(object):
         self.get_chassis_data_for_ml2_bind_port = mock.Mock()
         self.get_chassis_data_for_ml2_bind_port.return_value = \
             ('fake', '', ['fake-physnet'])
+        self.get_logical_port_chassis_and_datapath = mock.Mock()
+        self.get_logical_port_chassis_and_datapath.return_value = \
+            ('fake', 'fake-dp')
+        self.get_chassis_and_physnets = mock.Mock()
+        self.get_gateway_chassis_from_cms_options = mock.Mock()
 
 
 class FakeOvsdbTransaction(object):
