@@ -55,6 +55,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.fake_network = \
             fakes.FakeNetwork.create_one_network(attrs=network_attrs).info()
         self.fake_router_port = {'device_id': '',
+                                 'network_id': self.fake_network['id'],
                                  'device_owner': 'network:router_interface',
                                  'mac_address': 'aa:aa:aa:aa:aa:aa',
                                  'fixed_ips': [{'ip_address': '10.0.0.100',
@@ -68,7 +69,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             'networks': ['10.0.0.100/24'],
             'external_ids': {
                 ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'subnet-id',
-                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'}}
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
+                ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
+                utils.ovn_name(self.fake_network['id'])}}
         self.fake_router_ports = [self.fake_router_port]
         self.fake_subnet = {'id': 'subnet-id',
                             'ip_version': 4,
@@ -122,7 +125,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             'may_exist': True,
             'external_ids': {
                 ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'ext-subnet-id',
-                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'},
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
+                ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
+                utils.ovn_name(self.fake_network['id'])},
             'gateway_chassis': ['hv1']}
         self.fake_floating_ip_attrs = {'floating_ip_address': '192.168.0.10',
                                        'fixed_ip_address': '10.0.0.10'}
@@ -146,11 +151,11 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
                 ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                     self.fake_floating_ip['router_id'])}}
         self.l3_inst = directory.get_plugin(plugin_constants.L3)
-        self._start_mock(
+        self.nb_idl = self._start_mock(
             'networking_ovn.l3.l3_ovn.OVNL3RouterPlugin._ovn',
             new_callable=mock.PropertyMock,
             return_value=fakes.FakeOvsdbNbOvnIdl())
-        self._start_mock(
+        self.sb_idl = self._start_mock(
             'networking_ovn.l3.l3_ovn.OVNL3RouterPlugin._sb_ovn',
             new_callable=mock.PropertyMock,
             return_value=fakes.FakeOvsdbSbOvnIdl())
@@ -178,11 +183,11 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self._start_mock(
             'neutron.db.l3_db.L3_NAT_dbonly_mixin.delete_router',
             return_value={})
-        self._start_mock(
+        self.mock_candidates = self._start_mock(
             'networking_ovn.common.ovn_client.'
             'OVNClient.get_candidates_for_scheduling',
             return_value=[])
-        self._start_mock(
+        self.mock_schedule = self._start_mock(
             'networking_ovn.l3.l3_ovn_scheduler.'
             'OVNGatewayLeastLoadedScheduler._schedule_gateway',
             return_value=['hv1'])
@@ -245,7 +250,8 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             'fixed_ips': [
                 {'ip_address': '2001:db8::1', 'subnet_id': 'subnet-id1'},
                 {'ip_address': '2001:dba::1', 'subnet_id': 'subnet-id2'}],
-            'mac_address': 'aa:aa:aa:aa:aa:aa'
+            'mac_address': 'aa:aa:aa:aa:aa:aa',
+            'network_id': 'network-id1'
             }
         fake_rtr_intf_networks = ['2001:db8::1/24', '2001:dba::1/24']
         self.l3_inst.add_router_interface(self.context, router_id,
@@ -287,7 +293,9 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             networks=['10.0.0.100/24'],
             external_ids={
                 ovn_const.OVN_SUBNET_EXT_IDS_KEY: 'subnet-id',
-                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1'})
+                ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
+                ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
+                utils.ovn_name(self.fake_network['id'])})
 
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_router')
@@ -890,6 +898,19 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         self.l3_inst._ovn.delete_lswitch_port.assert_called_once_with(
             'fip-port-id', 'neutron-fip-net-id')
 
+    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
+    def test_create_floatingip_lsp_external_id(self, gf):
+        foo_lport = fakes.FakeOvsdbRow.create_one_ovsdb_row()
+        foo_lport.uuid = 'foo-port'
+        self.l3_inst._ovn.get_lswitch_port.return_value = foo_lport
+        self.l3_inst.create_floatingip(self.context, 'floatingip')
+        calls = [mock.call(
+            'Logical_Switch_Port',
+            'foo-port',
+            ('external_ids', {ovn_const.OVN_PORT_FIP_EXT_ID_KEY:
+                              '192.168.0.10'}))]
+        self.l3_inst._ovn.db_set.assert_has_calls(calls)
+
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.delete_floatingip')
     def test_delete_floatingip(self, df):
         self.l3_inst._ovn.get_floatingip.return_value = (
@@ -901,6 +922,35 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             logical_ip='10.0.0.10',
             external_ip='192.168.0.10')
 
+    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
+    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.delete_floatingip')
+    def test_delete_floatingip_lsp_external_id(self, df, gf):
+        gf.return_value = self.fake_floating_ip
+        self.l3_inst._ovn.get_floatingip.return_value = (
+            self.fake_ovn_nat_rule)
+
+        foo_lport = fakes.FakeOvsdbRow.create_one_ovsdb_row()
+        foo_lport.uuid = 'foo-port'
+        foo_lport.external_ids = {
+            ovn_const.OVN_PORT_FIP_EXT_ID_KEY: 'foo-port'}
+        self.l3_inst._ovn.get_lswitch_port.return_value = foo_lport
+
+        self.l3_inst.delete_floatingip(self.context, 'floatingip-id')
+        calls = [mock.call(
+            'Logical_Switch_Port', 'foo-port',
+            'external_ids', ovn_const.OVN_PORT_FIP_EXT_ID_KEY)]
+        self.l3_inst._ovn.db_remove.assert_has_calls(calls)
+
+    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
+    @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.delete_floatingip')
+    def test_delete_floatingip_no_lsp_external_id(self, df, gf):
+        gf.return_value = self.fake_floating_ip
+        self.l3_inst._ovn.get_floatingip.return_value = (
+            self.fake_ovn_nat_rule)
+        self.l3_inst._ovn.get_lswitch_port.return_value = None
+        self.l3_inst.delete_floatingip(self.context, 'floatingip-id')
+        self.l3_inst._ovn.db_remove.assert_not_called()
+
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_floatingip')
@@ -1005,14 +1055,33 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_floatingip')
-    def test_update_floatingip_association_not_changed(self, uf, gf):
-        self.fake_floating_ip.update({'fixed_port_id': None})
-        self.fake_floating_ip_new.update({'port_id': None})
+    def test_update_floatingip_association_empty_update(self, uf, gf):
+        self.l3_inst._ovn.is_col_present.return_value = True
+        self.l3_inst._ovn.get_floatingip.return_value = (
+            self.fake_ovn_nat_rule)
+        self.fake_floating_ip.update({'fixed_port_id': 'foo'})
+        self.fake_floating_ip_new.update({'port_id': 'foo'})
         gf.return_value = self.fake_floating_ip
         uf.return_value = self.fake_floating_ip_new
         self.l3_inst.update_floatingip(self.context, 'id', 'floatingip')
-        self.l3_inst._ovn.delete_nat_rule_in_lrouter.assert_not_called()
-        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_not_called()
+        self.l3_inst._ovn.delete_nat_rule_in_lrouter.assert_called_once_with(
+            'neutron-router-id',
+            type='dnat_and_snat',
+            logical_ip='10.0.0.10',
+            external_ip='192.168.0.10')
+        expected_ext_ids = {
+            ovn_const.OVN_FIP_EXT_ID_KEY: self.fake_floating_ip_new['id'],
+            ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
+            ovn_const.OVN_FIP_PORT_EXT_ID_KEY:
+                self.fake_floating_ip_new['port_id'],
+            ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
+                self.fake_floating_ip_new['router_id'])}
+        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+            'neutron-new-router-id',
+            type='dnat_and_snat',
+            logical_ip='10.10.10.10',
+            external_ip='192.168.0.10',
+            external_ids=expected_ext_ids)
 
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
@@ -1126,6 +1195,62 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
         mock_updt_status.assert_called_once_with(
             mock.ANY, fake_port_id, constants.PORT_STATUS_DOWN)
 
+    def test_schedule_unhosted_gateways_no_gateways(self):
+        self.nb_idl().get_unhosted_gateways.return_value = []
+        self.l3_inst.schedule_unhosted_gateways()
+        self.nb_idl().update_lrouter_port.assert_not_called()
+
+    def test_schedule_unhosted_gateways(self):
+        unhosted_gws = ['lrp-foo-1', 'lrp-foo-2', 'lrp-foo-3']
+        chassis_mappings = {
+            'chassis1': ['physnet1'],
+            'chassis2': ['physnet1'],
+            'chassis3': ['physnet1']}
+        chassis = ['chassis1', 'chassis2', 'chassis3']
+        self.sb_idl().get_chassis_and_physnets.return_value = (
+            chassis_mappings)
+        self.sb_idl().get_gateway_chassis_from_cms_options.return_value = (
+            chassis)
+        self.nb_idl().get_unhosted_gateways.return_value = unhosted_gws
+        # 1. port has 2 gateway chassis
+        # 2. port has only chassis2
+        # 3. port is not bound
+        existing_port_bindings = [
+            ['chassis1', 'chassis2'],
+            ['chassis2'],
+            []]
+        self.nb_idl().get_gateway_chassis_binding.side_effect = (
+            existing_port_bindings)
+        # for 1. port schedule untouched, add only 3'rd chassis
+        # for 2. port master scheduler somewhere else
+        # for 3. port schedule all
+        self.mock_schedule.side_effect = [
+            ['chassis1', 'chassis2', 'chassis3'],
+            ['chassis1', 'chassis2', 'chassis3'],
+            ['chassis3', 'chassis2', 'chassis1']]
+
+        self.l3_inst.schedule_unhosted_gateways()
+
+        self.mock_candidates.assert_has_calls([
+            mock.call(mock.ANY,
+                      chassis_physnets=chassis_mappings,
+                      cms=chassis)] * 3)
+        self.mock_schedule.assert_has_calls([
+            mock.call(self.nb_idl(), self.sb_idl(),
+                      'lrp-foo-1', [], ['chassis1', 'chassis2']),
+            mock.call(self.nb_idl(), self.sb_idl(),
+                      'lrp-foo-2', [], ['chassis2']),
+            mock.call(self.nb_idl(), self.sb_idl(),
+                      'lrp-foo-3', [], [])])
+        # make sure that for second port master chassis stays untouched
+        self.nb_idl().update_lrouter_port.assert_has_calls([
+            mock.call('lrp-foo-1',
+                      gateway_chassis=['chassis1', 'chassis2', 'chassis3']),
+            mock.call('lrp-foo-2',
+                      gateway_chassis=['chassis2', 'chassis1', 'chassis3']),
+            mock.call('lrp-foo-3',
+                      gateway_chassis=['chassis3', 'chassis2', 'chassis1'])])
+
 
 class OVNL3ExtrarouteTests(test_l3_gw.ExtGwModeIntTestCase,
                            test_l3.L3NatDBIntTestCase,
