@@ -26,6 +26,8 @@ from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import idlutils
 
 from networking_ovn.common import config as ovn_config
+from networking_ovn.common import constants as ovn_const
+from networking_ovn.common import utils
 from networking_ovn.ovsdb import ovsdb_monitor
 from networking_ovn.tests import base
 from networking_ovn.tests.unit import fakes
@@ -80,6 +82,10 @@ OVN_SB_SCHEMA = {
 }
 
 
+ROW_CREATE = ovsdb_monitor.BaseEvent.ROW_CREATE
+ROW_UPDATE = ovsdb_monitor.BaseEvent.ROW_UPDATE
+
+
 class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
 
     def setUp(self):
@@ -370,6 +376,61 @@ class TestOvnConnection(base.TestCase):
                                     schema='OVN_Southbound')
 
 
+class TestChassisMetadataAgentEvent(base.TestCase):
+
+    def setUp(self):
+        super(TestChassisMetadataAgentEvent, self).setUp()
+        self.event = ovsdb_monitor.ChassisMetadataAgentEvent()
+
+        patcher = mock.patch.object(ovsdb_monitor.stats, 'AgentStats')
+        self.agent_stats = patcher.start()
+        self.addCleanup(patcher.stop)
+
+    def test_run(self):
+        row = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'external_ids': {
+                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
+
+        self.event.run(mock.ANY, row, mock.ANY)
+        self.agent_stats.add_stat.assert_called_once_with(
+            utils.ovn_metadata_name(row.uuid), 1)
+
+    def test_run_no_metadata_cfg(self):
+        row = fakes.FakeOvsdbRow.create_one_ovsdb_row()
+        self.assertIsNone(self.event.run(mock.ANY, row, mock.ANY))
+
+    def test_match_fn_event_create(self):
+        self.assertTrue(self.event.match_fn(ROW_CREATE, mock.ANY))
+
+    def test_match_fn_missing_metadata_key(self):
+        row = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'external_ids': {
+                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
+        old = fakes.FakeOvsdbRow.create_one_ovsdb_row()
+
+        self.assertFalse(self.event.match_fn(ROW_UPDATE, row, old=old))
+
+    def test_match_fn_nb_cfg_match(self):
+        row = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'external_ids': {
+                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
+        old = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'external_ids': {
+                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
+
+        self.assertFalse(self.event.match_fn(ROW_UPDATE, row, old=old))
+
+    def test_match_fn_nb_cfg_doesnt_match(self):
+        row = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'external_ids': {
+                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '2'}})
+        old = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'external_ids': {
+                ovn_const.OVN_AGENT_METADATA_SB_CFG_KEY: '1'}})
+
+        self.assertTrue(self.event.match_fn(ROW_UPDATE, row, old=old))
+
+
 class TestPortBindingChassisUpdateEvent(base.TestCase):
     def setUp(self):
         super(TestPortBindingChassisUpdateEvent, self).setUp()
